"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class HttpClient {
    constructor(config = {}) {
        this.baseUrl = "";
        this.commonHeaders = {};
        this.commonParams = {};
        this.baseUrl = config.baseURL ?? "";
        this.commonHeaders = config.headers ?? {};
        this.commonParams = config.params ?? {};
    }
    static create(config = {}) {
        return new HttpClient(config);
    }
    deleteHeader(key) {
        delete this.commonHeaders[key];
    }
    setHeader(key, value) {
        this.commonHeaders[key] = value;
    }
    get(url, headers = {}, blob = false) {
        const params = headers?.params ?? null;
        let urlWithParams = url;
        if (params) {
            const qs = Object.entries(params)
                .map(([key, value]) => `${key}=${value}`)
                .join("&");
            urlWithParams = `${url}${url.includes("?") ? "&" : "?"}${qs}`;
            delete headers.params;
        }
        if (headers?.headers) {
            headers = headers.headers;
        }
        const request = fetch(this.getUrl(urlWithParams), {
            headers: this.getHeaders(headers),
        });
        return this.handleFetchResponse(urlWithParams, request, blob);
    }
    post(url, params = {}, extraHeaders = {}) {
        let body = JSON.stringify(params);
        const headers = this.getHeaders(extraHeaders);
        if (params instanceof FormData) {
            body = params;
            delete headers["content-type"];
        }
        const request = fetch(this.getUrl(url), {
            method: "post",
            body,
            headers,
            mode: "cors",
        });
        return this.handleFetchResponse(url, request);
    }
    patch(url, params = {}, headers = {}) {
        const request = fetch(this.getUrl(url), {
            method: "patch",
            body: JSON.stringify(params),
            headers: this.getHeaders(headers),
            mode: "cors",
        });
        return this.handleFetchResponse(url, request);
    }
    put(url, params = {}, headers = {}) {
        const request = fetch(this.getUrl(url), {
            method: "put",
            body: JSON.stringify(params),
            headers: this.getHeaders(headers),
            mode: "cors",
        });
        return this.handleFetchResponse(url, request);
    }
    delete(url, headers = {}) {
        const request = fetch(this.getUrl(url), {
            method: "delete",
            headers: this.getHeaders(headers),
        });
        return this.handleFetchResponse(url, request);
    }
    getUrl(url) {
        let result = this.baseUrl + url;
        if (this.baseUrl?.[this.baseUrl.length - 1] === "/" && url?.[0] === "/") {
            result = this.baseUrl + url.substring(1);
        }
        const paramsEntries = Object.entries(this.commonParams ?? {});
        if (paramsEntries.length) {
            const customParams = paramsEntries
                .map(([key, value]) => `${key}=${value}`)
                .join("&");
            if (result.includes("?")) {
                result += `&${customParams}`;
            }
            else {
                result += `?${customParams}`;
            }
        }
        return result;
    }
    getHeaders(headers = {}) {
        headers = headers?.headers ?? headers;
        return {
            "Access-Control-Allow-Origin": "*",
            ...this.commonHeaders,
            ...headers,
        };
    }
    // eslint-disable-next-line class-methods-use-this
    handleFetchResponse(url, request, blob = false) {
        return request
            .then(async (response) => {
            const data = await (blob ? response.blob() : response.json());
            // check for error response
            if (!response.ok || data?.error) {
                // get error message from body or default to response statusText
                const error = data?.metaData?.message || data?.message || response.statusText;
                // eslint-disable-next-line prefer-promise-reject-errors
                return Promise.reject({
                    message: error,
                    ...data,
                    response: { ...response, data },
                });
            }
            return Promise.resolve(blob ? data : { data });
        })
            .catch(error => {
            console.error(`<!!!> HTTP error route "${this.getUrl(url)}" with ${error.message || ""}`, error);
            return Promise.reject(error);
        });
    }
}
exports.default = HttpClient;
